from doit.tools import Interactive
from doit.action import CmdAction

from doit.tools import Interactive
from doit.action import CmdAction

dcc_params = [
    {
        'name':'compose_cmd',
        'short':'compose_cmd',
        'long': 'compose_cmd',
        'type': str,
        'default': 'docker-compose'
    },
    {
        'name':'dj_service',
        'short':'dj_service',
        'long': 'dj_service',
        'type': str,
        'default': 'django'
    }
]


def task_build():
    """ Build local docker images """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} build"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_up():
    """ Start local docker containers  """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} up"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_upd():
    """ Start local docker containers in detatched mode """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} up -d"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_logs():
    """ Follow docker logs """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} logs -f"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_stop():
    """ Stop docker containers """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} stop"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_down():
    """ Kill docker containers """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} down"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_attach():
    """ Create django superuser """

    def dcc(compose_cmd, dj_service):
        return f"{compose_cmd} attach {dj_service}"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2
    }


def task_pip():
    """ Run pip command. """

    def dcc(pos, compose_cmd, dj_service):
        arguments_space = " ".join(pos)
        return f"{compose_cmd} exec {dj_service} pip {pos}"

    return {
        "actions": [Interactive(dcc)],
        "params": dcc_params,
        'verbosity': 2,
        'pos_arg': 'pos'
    }


# TODO: translate to new format
# def task_manage():
#     """ run django manage.py command """

#     def argv(pos):
#         return f"{DJANGO_COMPOSE_CMD} run --rm {DJANGO_SERVICE_NAME} python manage.py {' '.join(pos)}"

#     return {
#         "actions": [Interactive(argv)],
#         "verbosity": 2,
#         "pos_arg": "pos"
#     }


# def task_shell():
#     """ Open django shell """

#     return {
#         "actions": [Interactive(f"{DJANGO_COMPOSE_CMD} run --rm {DJANGO_SERVICE_NAME} python manage.py shell_plus")],
#         "verbosity": 2
#     }


# def task_migrations():
#     """ Django make & run migrations """
#     return {
#         "actions": [
#             Interactive(f"{DJANGO_COMPOSE_CMD} run --rm {DJANGO_SERVICE_NAME} python manage.py makemigrations"),
#             Interactive(f"{DJANGO_COMPOSE_CMD} run --rm {DJANGO_SERVICE_NAME} python manage.py migrate")
#         ],
#         "verbosity": 2
#     }


# def task_format():
#     """ Format code """

#     return {
#         "actions": [
#             f"{DJANGO_COMPOSE_CMD} exec {DJANGO_SERVICE_NAME} isort --recursive apps",
#             f"{DJANGO_COMPOSE_CMD} exec {DJANGO_SERVICE_NAME} black -l 120 .",
#         ],
#         "verbosity": 2,
#     }


# def task_test():
#     """ Run pytest """

#     def pass_arguments(pos):
#         arguments_space = " ".join(pos)
#         return f"{DJANGO_COMPOSE_CMD} exec {DJANGO_SERVICE_NAME} pytest {arguments_space}"

#     return {"actions": [Interactive(pass_arguments)], "verbosity": 2, "pos_arg": "pos"}



from django.db import models


class PyProduct(models.Model):
    """ Product in shop.
    """
    sku = models.CharField(max_length=20, unique=True)
    name = models.CharField(max_length=200)
    short_description = models.CharField(blank=True, max_length=255)
    description = models.TextField(blank=True)

    class Meta:
        abstract = True

    def __str__(self):
        return f"({self.sku}) {self.name}"

class PyCategory(models.Model):
    """ Category of products.
    """
    name = models.CharField(max_length=255)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class PyBrand(models.Model):
    """ Product's Brand.
    """

    name = models.CharField(max_length=80)

    class Meta:
        abstract = True


class PyHomeSection(models.Model):
    """ Homescreen section.
    """

    name = models.CharField(max_length=80)
    order = models.PositiveIntegerField(default=0)

    class Meta:
        abstract = True



from azure.storage.file import FileService
from azure.storage.file.models import FilePermissions
from django.conf import settings
from django.utils import timezone

read_permission = FilePermissions(read=True)

class ClientAFS:
    """ Manage files on Azure File Storage.
        Default expiry time of SAS signature is set to one hour.
    """

    def __init__(self, account_name, account_key, expiry=3600):
        self.account_name = account_name
        self.client = FileService(account_name=account_name, account_key=account_key)
        self.expiry = expiry

    def get_file_sas(self, share_name, file_name):
        """ Fetch token for temporary public file read access.
        """
        return self.client.generate_file_shared_access_signature(
            share_name=share_name,
            file_name=file_name,
            permission=read_permission,
            expiry=timezone.now()+timezone.timedelta(seconds=self.expiry)
        )

    def public_file_link(self, share_name, file_name):
        return (f"https://{self.account_name}.file.core.windows.net/"
                f"{file_name}?{self.get_file_sas(share_name, file_name)}")



import os, sys
import setuptools

with open("README.md", "r") as _file:
    readme = _file.read()

setuptools.setup(
    name="appchance",
    version="0.1.20",
    author="Appchance Backend Special Forces",
    author_email="backend@appchance.com",
    description="Appchance's toolbelt for wizards and ninjas. May be useful using in dungeons.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/appchance/pychance/",
    packages=setuptools.find_packages(),
    install_requires=["doit"],
    scripts=["bin/dodo"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Framework :: Django :: 3.0",
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 2 - Pre-Alpha",
        "Operating System :: POSIX :: Linux"
    ],
    python_requires='>=3',
)
